preset_settings = {
    'tdm': {
        'Server': {
            "Bots" : "0",
            "Gamemode": "2",
            "MapSelection": "1",
            "Hazards": "0",
            "Powerups": "0",
            "Name": "Team Deathmatch"
        },
        'ServerMutators': {
            "CaptEnable": "1",
        }
    },
    'dm': {
        'Server': {
            "Bots" : "3",
            "Gamemode": "0",
            "MapSelection": "1",
            "Hazards": "0",
            "Powerups": "1",
            "Name": "Deathmatch"
        }
    },
}


import fire
import docker

from cli_utils import (
    get_env,
    get_ports,
    get_all_servers,
    convert_toml_to_dict,
    post_new_server,
    server_is_running,
    delete_server
)
from presets import preset_settings

def requires_server(func):
    def wrapper(*args, **kwargs):
        
        if server_is_running():
            return func(*args, **kwargs)
        else:
            print("""
            Server is not running. 
            [python cli.py start] to get started
            """)
    return wrapper


class BoringCLI(object):
    """Tools for managing boring man servers."""

    def start(self, num_servers=3, base_port=42069):
        docker_env = get_env(num_servers, base_port)
        client = docker.from_env()
        try:
            container = client.containers.run(
                "coyotebm/bm-server-api:dev", 
                environment=get_env(base_port, num_servers),
                ports=get_ports(base_port, num_servers),
                detach=True
            )
        except docker.errors.APIError:
            print("An error occurred and the server could not start. Is it already running?")
            return
        with open(".container_id", 'w') as container_file:
            container_file.write(container.id)
        print(f'Started Server listening on 8000')


    def stop(self):
        """Kills the currently running server and all boring man servers"""
        with open('.container_id', 'r') as container_file:
            container_id = container_file.readline()
            client = docker.from_env()
            try:
                container = client.containers.get(container_id)
                container.kill()
            except docker.errors.APIError:
                print("Server is not running")

    @requires_server
    def status(self):
        """Gets the current status of the running server"""
        print(get_all_servers(8000))


    @requires_server
    def add_custom(self, settings_file_path: str):
        """Creates a custom server from an ini file"""
        settings = convert_toml_to_dict(settings_file_path)
        response = post_new_server(settings)
        print(
            f'Added a new server. ID: {response["game_id"]}'
        )

    @requires_server
    def remove(self, game_id: int):
        """Stops a boring man server, freeing it's space"""
        # import pdb; pdb.set_trace()
        response = delete_server(int(game_id))
        if 'error' in response:
            print(response['error'])
        if 'status' in response:
            print(f'Remove {response["game_id"]}: {response["status"]}')


    @requires_server
    def add(self, server_preset: str):
        """Adds a preset server"""
        if server_preset in preset_settings:
            response = post_new_server(preset_settings[server_preset])
            print(
                f'Added a new server. ID: {response["game_id"]}'
            )
        else:
            print(f'Not a preset, valid options are {",".join(preset_settings.keys())}')
            

def main():
    fire.Fire(BoringCLI)

if __name__ == '__main__':
    main()


from collections import defaultdict
import configparser
import json

from tabulate import tabulate
import requests
import toml

def _get_port_list(base_port: int, num_servers: int):
    """Generate a list of ports"""
    return [str(2 * x + base_port) for x in range(num_servers)]


def get_env(base_port: int, num_servers: int):
    """Returns the environment needed to run the docker file"""
    game_ports = _get_port_list(base_port, num_servers)
    rcon_ports = _get_port_list(base_port + 1, num_servers)
    return {
        'GAME_PORTS': ','.join(game_ports),
        'RCON_PORTS': ','.join(rcon_ports)
    }


def get_ports(base_port: int, num_servers: int):
    """Returns the forwarded ports"""
    all_ports = 2 * num_servers
    tcp_ports = {'{}/tcp'.format(i) : i for i in range(base_port, base_port + all_ports)}
    udp_ports = {'{}/udp'.format(i) : i for i in range(base_port, base_port + all_ports)}
    return {
        **tcp_ports,
        **udp_ports,
        '8000/tcp': 8000
    }


def get_all_servers(server_port):
    """Gets all the currently running services and pretty prints them"""
    server_list = requests.get(f'http://127.0.0.1:{server_port}').json()['servers']
    active_servers = [ server for server in server_list if server['game_id'] != None ]
    if (len(active_servers) == 0):
        return "There are no servers running, add one with the add command"
    else:
        header = active_servers[0].keys()
        rows = [x.values() for x in active_servers]
        return tabulate(rows, header)

def server_is_running(server_port=8000):
    """Returns true if the server is running, otherwise false"""
    try:
        response = requests.get(f'http://127.0.0.1:{server_port}')
    except Exception as e:
        return False
    return True


def convert_to_dict(config):
    config_dict = defaultdict(dict)
    for section in config.sections():
        for key, value in config.items(section):
            config_dict[section][key] = value.strip('"')
    return dict(config_dict)


def convert_toml_to_dict(toml_settings_path: str):
    """Converts a toml string to python dictionary"""
    config = configparser.ConfigParser()
    # Override key format. This keeps key case unchanged
    config.optionxform = str
    config.read(toml_settings_path)
    return convert_to_dict(config)


def post_new_server(settings_dict: dict, mode = 'normal'):
    """Posts a new bman server"""
    settings_dict['mode'] = mode
    serialized_settings = json.dumps(settings_dict)
    serialized_settings.replace("'", '"')
    serialized_settings.replace("////", "/")
    headers = {
        'Content-Type': 'application/json'
    }

    response = requests.post(
        'http://127.0.0.1:8000',
        headers=headers,
        data=serialized_settings)
    return response.json()


def delete_server(game_id: int):
    """Deletes a running boring man server"""
    response = requests.delete(
        'http://127.0.0.1:8000/' + str(game_id),
    )
    return response.json()